@@ -46,6 +46,7 @@ void errorcall (SEXP call, const char *format, ...);
 int hdf5_global_verbosity = 3;
 int hdf5_global_nametidy = 0;
 int hdf5_global_attrcnt;
+const char *specificDataset;
 
 #if ((H5_VERS_MAJOR > 1) || (H5_VERS_MAJOR == 1 && H5_VERS_MINOR > 6) || (H5_VERS_MAJOR == 1 && H5_VERS_MINOR == 6 && H5_VERS_RELEASE >= 3)) && 1
 #define count_size_t size_t
@@ -67,6 +68,15 @@ nametidy (char *name)
   for (i = 1; i < strlen (name); i++)
     if(!isalnum (name[i]) && name[i] !='.')
       name[i] = '.';
+      
+  if (strlen(name) == 1)
+  {
+    if ( (name[0] == 'D') || (name[0] == 'T') || (name[0] == 'C') || (name[0] == 'F') || (name[0] == 'I')
+      || (name[0] == 'q') || (name[0] == 't') || (name[0] == 'c'))
+    {
+      sprintf(name, "%s_", name);
+    }
+  }
 }
 
 static herr_t
@@ -192,9 +202,9 @@ permute (struct permute_info *pinfo, unsigned dimnum)
 	  {
 	    if (pinfo->writeflag)
               {
-	        char **tmpaddr;
+	        const char **tmpaddr;
 
-	        tmpaddr = &((char **) pinfo->tmpbuf)[pinfo->tmpcnt];
+	        tmpaddr = &((const char **) pinfo->tmpbuf)[pinfo->tmpcnt];
 	        *tmpaddr = CHAR (STRING_ELT (pinfo->buf, offset));
               }
 	    else
@@ -202,7 +212,6 @@ permute (struct permute_info *pinfo, unsigned dimnum)
 		char *ptr =
 		  ((char **) pinfo->tmpbuf)[pinfo->tmpcnt];
 	
-		Rprintf ("[%s]\n", ptr);
 		SET_STRING_ELT (pinfo->buf, offset, mkChar (ptr));
 	      }
 	  }
@@ -1101,7 +1110,7 @@ do_hdf5save (SEXP args)
       if (TYPEOF (CAR (names)) != STRSXP)
         errorcall (call, "expecting a symbol name");
       {
-        char *name = CHAR (STRING_ELT (CAR (names), 0));
+        const char *name = CHAR (STRING_ELT (CAR (names), 0));
         
         PROTECT (sym = install (name));
         PROTECT (s = findVar (sym, env));
@@ -1127,7 +1136,8 @@ static void
 add_to_list (struct hdf5_iterate_info *iinfo, const char *name, SEXP obj)
 {
   SEXP pair;
-  char newname[strlen(name)+1];
+  char newname[strlen(name)+16];
+  memset(newname, '\0', strlen(name)+16);
 
   strcpy (newname,name);
   if (hdf5_global_nametidy)
@@ -1284,7 +1294,8 @@ hdf5_process_attribute (hid_t loc_id, const char *attrName, void *data)
   hid_t aid, sid, tid;
   H5T_class_t class;
   size_t tid_size;
-  char newname[strlen(attrName) +1];
+  char newname[strlen(attrName) +16];
+  memset(newname, '\0', strlen(attrName)+16);
 
   if (strcmp (attrName, ROWNAMES) == 0)
     {
@@ -1496,6 +1507,11 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
 
   if (hdf5_global_verbosity > 0)
     Rprintf ("Processing object: %s ...",name);
+  if ( (specificDataset != NULL) && (strcmp(name, specificDataset) != 0) )
+  {
+    Rprintf (" is not going to be processed (Searching for %s) \n", specificDataset);
+    return 0;
+  }
   if (H5Gget_objinfo (id, name, 1, &statbuf) < 0)
     errorcall (iinfo->call, "Cannot query object `%s'", name);
 
@@ -1811,7 +1827,8 @@ add_to_symbol_table (struct hdf5_iterate_info *iinfo,
 		     const char *name,
 		     SEXP obj)
 {
-  char newname[strlen(name)+1];
+  char newname[strlen(name)+16];
+  memset(newname, '\0', strlen(name)+16);
   strcpy(newname,name);
   if(hdf5_global_nametidy)
     {
@@ -1827,6 +1844,7 @@ add_to_symbol_table (struct hdf5_iterate_info *iinfo,
 SEXP do_hdf5load (SEXP args)
 {
   const char *path;
+  specificDataset = NULL;
   hid_t fid;
   int restore_syms;
   struct hdf5_iterate_info iinfo;
@@ -1852,6 +1870,15 @@ SEXP do_hdf5load (SEXP args)
   hdf5_global_nametidy  = INTEGER (CADDDR (args))[0];
   H5dont_atexit ();
   
+  args = CDR (args); args = CDR (args); args = CDR (args);
+  args = CDR (args);
+  if (isValidString(CAR(args)))
+  {
+    specificDataset = CHAR (STRING_ELT (CAR(args), 0));
+    if (strlen(specificDataset) == 0)
+      specificDataset = NULL;
+  }
+  
   if ((fid = H5Fopen (path, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0)
     errorcall (call, "unable to open HDF file: %s", path);
 
